-- name: CreateEvent :execlastid
INSERT INTO events
    (title, description, creator, location, has_time, start_time, end_time, created_at)
VALUES
    (?, ?, ?, ?, ?, ?, ?, datetime('now'));

-- name: GetEvent :one
SELECT * FROM events
WHERE id = ?;

-- name: GetEventPhoto :one
SELECT * FROM event_photos
WHERE event_id = ?
LIMIT 1;

-- name: GetEventTags :many
SELECT t.*
FROM event_tags et
JOIN tags t ON et.tag_id = t.id
WHERE et.event_id = ?;

-- name: GetRecentlyCreatedEvents :many
SELECT * FROM events
ORDER BY date(created_at) DESC
LIMIT 10;

-- name: GetUpcomingEventsInRange :many
SELECT * FROM events
WHERE 
    date(start_time) >= date('now', 'localtime', 'start of day')
    AND date(start_time, 'localtime') >= date(?, 'start of day')
    AND date(start_time, 'localtime') <= date(?, 'start of day')
ORDER BY start_time ASC;

-- name: GetUpcomingEventsByTag :many
SELECT e.* FROM events e
JOIN event_tags et ON e.id = et.event_id
WHERE
    date(e.start_time) >= date('now', 'localtime', 'start of day')
    AND et.tag_id = ?
ORDER BY start_time ASC;

-- name: UpdateEvent :exec
UPDATE events SET
    title = ?,
    description = ?,
    location = ?,
    has_time = ?,
    start_time = ?,
    end_time = ?
WHERE id = ?;

-- name: DeleteEventPhoto :exec
DELETE FROM event_photos
WHERE event_id = ?;

-- name: SetEventPhoto :exec
INSERT INTO event_photos
    (event_id, created_at, preview_jpg)
VALUES
    (?, datetime('now'), ?);

-- name: AddEventTag :exec
INSERT INTO event_tags
    (event_id, tag_id)
VALUES
    (?, ?);

-- name: RemoveEventTag :exec
DELETE FROM event_tags
WHERE event_id = ? AND tag_id = ?;

-- name: DeleteEvent :exec
-- DeleteEvent completely removes an event.
DELETE FROM events
WHERE id = ?;
