package admin

import (
	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/middleware"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services"
)

type adminCtr struct {
	controller.Controller
}

func BuildRoutes(c *services.Container, g *echo.Group, ctr controller.Controller) {
	sg := g.Group("/admin", middleware.RequireAdmin(c.DB, c.Enforcer))

	adminCtr := adminCtr{Controller: ctr}

	sg.GET("", adminCtr.GetHome).Name = pages.Names.Admin.Home
	sg.GET("/users", adminCtr.GetUsers).Name = pages.Names.Admin.Users
	sg.GET("/users/:id", adminCtr.GetUserDetails).Name = pages.Names.Admin.UserDetails
	sg.PATCH("/users/:id", adminCtr.PatchUserDetails)
	sg.GET("/users/:id/delete", adminCtr.GetDeleteUser).Name = pages.Names.Admin.UserDelete
	sg.POST("/users/:id/delete", adminCtr.PostDeleteUser)

	sg.GET("/groups", adminCtr.GetGroups).Name = pages.Names.Admin.Groups
	sg.POST("/groups", adminCtr.PostGroups)
	sg.GET("/groups/:id", adminCtr.GetGroupDetails).Name = pages.Names.Admin.GroupDetails
	sg.POST("/groups/:id", adminCtr.PostGroupDetails)
	sg.GET("/groups/:id/delete", adminCtr.GetDeleteGroup).Name = pages.Names.Admin.GroupDelete
	sg.POST("/groups/:id/delete", adminCtr.PostDeleteGroup)
	sg.GET("/groups/:id/delete_user/:user_id", adminCtr.GetRemoveGroupUser).Name = pages.Names.Admin.GroupUserRemove
	sg.POST("/groups/:id/delete_user/:user_id", adminCtr.PostRemoveGroupUser)

	sg.GET("/tags", adminCtr.GetTags).Name = pages.Names.Admin.Tags
	sg.POST("/tags", adminCtr.PostTags)
	sg.GET("/tags/:id", adminCtr.GetTagDetails).Name = pages.Names.Admin.TagDetails
	sg.GET("/tags/:id/delete", adminCtr.GetDeleteTag).Name = pages.Names.Admin.TagDelete
	sg.POST("/tags/:id/delete", adminCtr.PostDeleteTag)
}
