package views

import (
	"io"
	"net/http"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type AdminGroupDetailsForm struct {
	AddUserEmail string `form:"email" valid:"required,email"`

	Submission controller.FormSubmission
}

type AdminGroupDetails struct {
	Group   sqlc.Group
	Members []sqlc.User
	Form    AdminGroupDetailsForm

	GroupListOnly bool
	Page          controller.Page
}

func (v *AdminGroupDetails) Render(w io.Writer) error {
	if v.GroupListOnly {
		return v.userTable().Render(w)
	}

	content := c.NarrowContainer(
		dividedSections(
			g.Group{
				c.PrimaryHeader("Group Details"),
				c.DescriptionList(
					h.Class("mt-2"),
					c.MakeTwoColumnRow(
						c.MakeCell("Name", g.Text(v.Group.Name)),
						nil),
				),
			},
			g.Group{
				c.TextLarge("Group Members"),
				c.HtmxSearch("#user-list", v.Page.Path),
				v.userTable(),
			},
			g.Group{
				c.FullScreenForm(
					http.MethodPost,
					v.Page.Path,
					"",
					"",
					c.FormFooter(nil, "", "Add user", false),
					c.FormSection(
						c.FormHeading("Add member", ""),
						c.CSRF(v.Page),
						c.InputText(c.FormInputParams{
							Name:     "email",
							Display:  "Email",
							Required: true,
							Value:    v.Form.AddUserEmail,
							Errors:   v.Form.Submission.GetFieldErrors("AddUserEmail"),
						}),
					)),
			},
			g.Group{
				// TODO: Align right
				c.ButtonDestructive(v.Page.ToURL(pages.Names.Admin.GroupDelete, v.Group.ID), g.Text("Delete Group")),
			},
		),
	)

	l := layouts.Primary{
		Header: c.HeaderContainer(
			c.Breadcrumbs([]c.BreadcrumbPart{
				{
					Display: "Admin",
					Href:    v.Page.ToURL(pages.Names.Admin.Home),
				},
				{
					Display: "Groups",
					Href:    v.Page.ToURL(pages.Names.Admin.Groups),
				},
				{
					Display: v.Group.Name,
				},
			},
			),
			nil,
		),
		Content: content,
		Page:    v.Page,
	}

	return l.Render(w)
}

func (v *AdminGroupDetails) userTable() g.Node {
	return h.Div(
		h.ID("user-list"),
		h.Table(
			h.Class("min-w-full divide-y divide-gray-300"),
			h.THead(
				h.Tr(
					h.Th(
						h.Class("py-3.5 pl-4 pr-3 text-left text-sm font-semibold text-gray-900 sm:pl-0"),
						g.Text("Email")),
					h.Th(
						h.Class("hidden px-3 py-3.5 text-left text-sm font-semibold text-gray-900 lg:table-cell"),
						g.Text("Name")),
					h.Th(
						h.Class("py-3.5 pl-4 pr-3 text-center text-sm font-semibold text-gray-900 sm:pl-0"),
						g.Text("Remove")))),
			h.TBody(
				h.Class("divide-y divide-gray-200 bg-white"),
				g.Map(v.Members, v.userRow))))
}

func (v *AdminGroupDetails) userRow(u sqlc.User) g.Node {
	return h.Tr(
		h.Td(
			h.Class("w-full max-w-0 py-4 pl-4 pr-3 text-sm font-medium text-gray-900 sm:w-auto sm:max-w-none sm:pl-0"),
			h.A(
				h.Class("underline"),
				h.Href(v.Page.ToURL(pages.Names.Admin.UserDetails, u.ID)),
				g.Text(u.Email)),
			h.Dl(
				h.Class("font-normal lg:hidden"),
				c.LazyIf(u.Name.Valid,
					func() g.Node {
						return g.Group{
							h.Dt(
								h.Class("sr-only"),
								g.Text("Name")),
							h.Dd(
								h.Class("mt-1 truncate text-gray-700"),
								g.Text(u.Name.String)),
						}
					}))),
		h.Td(
			h.Class("hidden px-3 py-4 text-sm text-gray-500 lg:table-cell"),
			g.Text(u.Name.String)),
		h.Td(
			h.Class("px-3 py-4 text-sm text-gray-500 flex justify-center"),
			h.A(
				h.Href(v.Page.ToURL(pages.Names.Admin.GroupUserRemove, v.Group.ID, u.ID)),
				c.IconX())))
}
