package views

import (
	"io"
	"net/http"

	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
	g "maragu.dev/gomponents"
)

type AdminAddGroupForm struct {
	Name string `form:"name" valid:"required"`

	Submission controller.FormSubmission
}

type AdminGroups struct {
	Groups        []sqlc.Group
	GroupListOnly bool
	Form          AdminAddGroupForm

	Page controller.Page
}

func (v *AdminGroups) Render(w io.Writer) error {
	if v.GroupListOnly {
		return v.groupTable().Render(w)
	}

	content := c.NarrowContainer(
		c.PrimaryHeader("Groups"),
		c.HtmxSearch("#group-list", v.Page.Path),
		v.groupTable(),
		v.addGroupForm())

	l := layouts.Primary{
		Header: c.HeaderContainer(
			c.Breadcrumbs([]c.BreadcrumbPart{
				{
					Display: "Admin",
					Href:    v.Page.ToURL(pages.Names.Admin.Home),
				},
				{
					Display: "Groups",
				},
			},
			),
			nil,
		),
		Content: content,
		Page:    v.Page,
	}

	return l.Render(w)
}

func (v *AdminGroups) groupTable() g.Node {
	return h.Div(
		h.ID("group-list"),
		h.Class("mb-4"),
		h.Table(
			h.Class("min-w-full divide-y divide-gray-300"),
			h.THead(
				h.Tr(
					h.Th(
						h.Class("py-3.5 pl-4 pr-3 text-left text-sm font-semibold text-gray-900 sm:pl-0"),
						g.Text("Name")),
					h.Th(
						h.Class("relative py-3.5 pl-3 pr-4 sm:pr-0"),
						h.Span(
							h.Class("sr-only"),
							g.Text("Edit"))))),
			h.TBody(
				h.Class("divide-y divide-gray-200 bg-white"),
				g.Map(v.Groups, v.groupRow))))
}

func (v *AdminGroups) groupRow(group sqlc.Group) g.Node {
	return h.Tr(
		h.Td(
			h.Class("w-full max-w-0 py-4 pl-4 pr-3 text-sm font-medium text-gray-900 sm:w-auto sm:max-w-none sm:pl-0"),
			h.A(
				h.Class("underline"),
				h.Href(v.Page.ToURL(pages.Names.Admin.GroupDetails, group.ID)),
				g.Text(group.Name)),
		))
}

func (v *AdminGroups) addGroupForm() g.Node {
	return c.FullScreenForm(
		http.MethodPost,
		v.Page.Path,
		"",
		"",
		c.FormFooter(nil, "", "Create", false),
		c.FormSection(
			c.FormHeading("Add Group", ""),
			c.CSRF(v.Page),
			c.InputText(c.FormInputParams{
				Name:     "name",
				Display:  "Name",
				Required: true,
				Value:    v.Form.Name,
				Errors:   v.Form.Submission.GetFieldErrors("Name"),
			}),
		))
}
