package views

import (
	"io"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type AdminTagDetails struct {
	Tag             sqlc.Tag
	RestrictedGroup *sqlc.Group

	Page controller.Page
}

func (v *AdminTagDetails) Render(w io.Writer) error {
	content := c.NarrowContainer(
		dividedSections(
			g.Group{
				c.PrimaryHeader("Tag Details"),
				c.DescriptionList(
					h.Class("mt-2"),
					c.MakeTwoColumnRow(
						c.MakeCell("Name", g.Text(v.Tag.Name)),
						c.MakeCell(
							"Restricted",
							c.LazyIfElse(v.RestrictedGroup == nil,
								func() g.Node {
									return g.Text("No restrictions")
								},
								func() g.Node {
									return c.TextLink(
										v.Page.ToURL(pages.Names.Admin.GroupDetails, v.RestrictedGroup.ID),
										g.Text(v.RestrictedGroup.Name))
								})))),
			},
			g.Group{
				// TODO: Align right
				c.ButtonDestructive(v.Page.ToURL(pages.Names.Admin.TagDelete, v.Tag.ID), g.Text("Delete Tag")),
			},
		),
	)

	l := layouts.Primary{
		Header: c.HeaderContainer(
			c.Breadcrumbs([]c.BreadcrumbPart{
				{
					Display: "Admin",
					Href:    v.Page.ToURL(pages.Names.Admin.Home),
				},
				{
					Display: "Tags",
					Href:    v.Page.ToURL(pages.Names.Admin.Tags),
				},
				{
					Display: v.Tag.Name,
				},
			},
			),
			nil,
		),
		Content: content,
		Page:    v.Page,
	}

	return l.Render(w)
}
