package views

import (
	"io"
	"net/http"
	"strconv"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"
)

type AdminAddTagForm struct {
	Name              string `form:"name" valid:"required"`
	RestrictedGroupID string `form:"restricted_group" valid:"optional"`

	Submission controller.FormSubmission
}

type AdminTags struct {
	Tags   []sqlc.Tag
	Groups []sqlc.Group
	Form   AdminAddTagForm
	Page   controller.Page
}

func (v *AdminTags) Render(w io.Writer) error {
	content := c.NarrowContainer(
		c.PrimaryHeader("Tags"),
		v.tagTable(),
		v.addTagForm())

	l := layouts.Primary{
		Header: c.HeaderContainer(
			c.Breadcrumbs([]c.BreadcrumbPart{
				{
					Display: "Admin",
					Href:    v.Page.ToURL(pages.Names.Admin.Home),
				},
				{
					Display: "Tags",
				},
			},
			),
			nil,
		),
		Content: content,
		Page:    v.Page,
	}

	return l.Render(w)
}

func (v *AdminTags) tagTable() g.Node {
	return h.Div(
		h.ID("tag-list"),
		h.Class("mb-4"),
		h.Table(
			h.Class("min-w-full divide-y divide-gray-300"),
			h.THead(
				h.Tr(
					h.Th(
						h.Class("py-3.5 pl-4 pr-3 text-left text-sm font-semibold text-gray-900 sm:pl-0"),
						g.Text("Name")),
					h.Th(
						h.Class("relative py-3.5 pl-3 pr-4 sm:pr-0"),
						h.Span(
							h.Class("sr-only"),
							g.Text("Edit"))))),
			h.TBody(
				h.Class("divide-y divide-gray-200 bg-white"),
				g.Map(v.Tags, v.tagRow))))
}

func (v *AdminTags) tagRow(tag sqlc.Tag) g.Node {
	return h.Tr(
		h.Td(
			h.Class("w-full max-w-0 py-4 pl-4 pr-3 text-sm font-medium text-gray-900 sm:w-auto sm:max-w-none sm:pl-0"),
			h.A(
				h.Class("underline"),
				h.Href(v.Page.ToURL(pages.Names.Admin.TagDetails, tag.ID)),
				g.Text(tag.Name)),
		))
}

func (v *AdminTags) addTagForm() g.Node {
	// Allow selecting no restriction by adding an empty option.
	options := make([]c.FormSelectOption, 0, len(v.Groups)+1)
	options = append(options, c.FormSelectOption{
		Display: "No restrictions",
		Value:   "",
	})
	for _, g := range v.Groups {
		options = append(options, c.FormSelectOption{
			Display: g.Name,
			Value:   strconv.Itoa(g.ID),
		})
	}

	return c.FullScreenForm(
		http.MethodPost,
		v.Page.Path,
		"",
		"",
		c.FormFooter(nil, "", "Create", false),
		c.FormSection(
			c.FormHeading("Add Tag", ""),
			c.CSRF(v.Page),
			c.InputText(c.FormInputParams{
				Name:     "name",
				Display:  "Name",
				Required: true,
				Value:    v.Form.Name,
				Errors:   v.Form.Submission.GetFieldErrors("Name"),
			}),
			c.InputSelect(c.FormSelectParams{
				FormInputParams: c.FormInputParams{
					Name:     "restricted_group",
					Display:  "Restricted Group",
					Required: false,
					Value:    v.Form.RestrictedGroupID,
					Errors:   v.Form.Submission.GetFieldErrors("RestrictedGroupID"),
				},
				Options: options,
			}),
		))
}
