package views

import (
	"fmt"
	"io"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type AdminUsers struct {
	Users        []sqlc.User
	UserListOnly bool
	Page         controller.Page
}

// TODO: Add sorting
// - Bounces
// - Most recent login
// - Most recent page load
// - Most recently created

func (v *AdminUsers) Render(w io.Writer) error {
	if v.UserListOnly {
		return v.userTable().Render(w)
	}

	content := c.NarrowContainer(
		c.PrimaryHeader("Users"),
		c.HtmxSearch("#user-list", v.Page.Path),
		v.userTable())

	l := layouts.Primary{
		Header: c.HeaderContainer(
			c.Breadcrumbs([]c.BreadcrumbPart{
				{
					Display: "Admin",
					Href:    v.Page.ToURL(pages.Names.Admin.Home),
				},
				{
					Display: "Users",
				},
			},
			),
			nil,
		),
		Content: content,
		Page:    v.Page,
	}

	return l.Render(w)
}

func (v *AdminUsers) userTable() g.Node {
	var moreNotice g.Node
	unshownUsers := v.Page.Pager.Items - len(v.Users)
	if unshownUsers > 0 {
		moreNotice = c.TextDescription(fmt.Sprintf("%d users not shown", unshownUsers))
	}

	return h.Div(
		h.ID("user-list"),
		h.Table(
			h.Class("min-w-full divide-y divide-gray-300"),
			h.THead(
				h.Tr(
					h.Th(
						h.Class("py-3.5 pl-4 pr-3 text-left text-sm font-semibold text-gray-900 sm:pl-0"),
						g.Text("Email")),
					h.Th(
						h.Class("hidden px-3 py-3.5 text-left text-sm font-semibold text-gray-900 lg:table-cell"),
						g.Text("Name")),
					h.Th(
						h.Class("py-3.5 pl-4 pr-3 text-center text-sm font-semibold text-gray-900 sm:pl-0"),
						g.Text("Verified")),
					h.Th(
						h.Class("relative py-3.5 pl-3 pr-4 sm:pr-0"),
						h.Span(
							h.Class("sr-only"),
							g.Text("Edit"))))),
			h.TBody(
				h.Class("divide-y divide-gray-200 bg-white"),
				g.Map(v.Users, v.userRow))),
		moreNotice)
}

func (v *AdminUsers) userRow(u sqlc.User) g.Node {
	return h.Tr(
		h.Td(
			h.Class("w-full max-w-0 py-4 pl-4 pr-3 text-sm font-medium text-gray-900 sm:w-auto sm:max-w-none sm:pl-0"),
			h.A(
				h.Class("underline"),
				h.Href(v.Page.ToURL(pages.Names.Admin.UserDetails, u.ID)),
				g.Text(u.Email)),
			h.Dl(
				h.Class("font-normal lg:hidden"),
				c.LazyIf(u.Name.Valid,
					func() g.Node {
						return g.Group{
							h.Dt(
								h.Class("sr-only"),
								g.Text("Name")),
							h.Dd(
								h.Class("mt-1 truncate text-gray-700"),
								g.Text(u.Name.String)),
						}
					}))),
		h.Td(
			h.Class("hidden px-3 py-4 text-sm text-gray-500 lg:table-cell"),
			g.Text(u.Name.String)),
		h.Td(
			h.Class("px-3 py-4 text-sm text-gray-500 flex justify-center"),
			c.LazyIf(u.Verified,
				func() g.Node {
					return c.IconCheck()
				})))
}
