package views

import (
	"fmt"
	"io"
	"time"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type EventList struct {
	Tag            sqlc.Tag
	UpcomingEvents []sqlc.Event
	EventListOnly  bool
	Page           controller.Page
}

func (v *EventList) Render(w io.Writer) error {
	if v.EventListOnly {
		return v.eventListItems().Render(w)
	}

	content := c.WideContainer(
		h.Div(
			h.Class("mx-auto max-w-2xl py-4 px-4 sm:py-6 sm:px-6 lg:max-w-7xl lg:px-8"),
			h.Div(
				h.Class("lg:pl-6"),
				c.PrimaryHeader(v.Tag.Name+" Events")),
			v.eventList(),
		),
	)

	l := layouts.Primary{
		Header:  c.EventHeader(v.Page),
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}

func (v *EventList) eventList() g.Node {
	if len(v.UpcomingEvents) == 0 {
		return h.Div(
			h.Class("flex flex-col items-center justify-center py-6 lg:mt-24"),
			h.Div(
				h.Class("text-gray-500"),
				c.IconCalendarDays()),
			c.TextLarge("There are no upcoming events for this tag."))
	}

	return h.Div(
		h.Class("divide-y divide-gray-100 text-sm leading-6 flex flex-col mt-4"),
		v.eventListItems())
}

// eventListItems does not have a wrapping container, just the items and optionally a load-more button.
func (v *EventList) eventListItems() g.Node {
	return g.Group{
		g.Group(g.Map(v.UpcomingEvents, v.EventListItem)),
		c.LazyIf(!v.Page.Pager.IsEnd(),
			func() g.Node {
				return h.Button(
					h.Class("inline-flex items-center justify-center gap-x-2 rounded-md border border-transparent bg-indigo-600 mx-auto mt-4 px-3 py-2 font-medium leading-4 text-white shadow-sm hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2 sm:text-sm"),
					c.HxGet(fmt.Sprintf("%s?page=%d", v.Page.Path, v.Page.Pager.Page+1)),
					c.HxTarget("this"),
					c.HxSwap("outerHTML"),
					c.HxTrigger("click"),
					g.Text("Load more events"))
			}),
	}
}

func (v *EventList) EventListItem(e sqlc.Event) g.Node {
	return h.Li(
		h.Class("relative flex space-x-6 py-6 px-6 hover:bg-gray-50 xl:static"),
		h.A(
			h.Href(v.Page.ToURL(pages.Names.Event.One, e.ID)),
			h.Class("flex justify-between items-center min-w-full"),
			h.Div(
				h.Class("flex-auto overflow-hidden"),
				c.TertiaryHeader(e.Title),
				c.LazyIf(v.Page.IsAuth, func() g.Node {
					dateTime := e.StartTime.In(v.Page.Cfg.App.Loc).Format("Mon Jan _2")
					if e.HasTime {
						dateTime += " at " + e.StartTime.In(v.Page.Cfg.App.Loc).Format(time.Kitchen)
					}

					return h.Dl(
						h.Class("mt-2 flex flex-col text-gray-500 xl:flex-row"),
						h.Div(
							h.Class("flex items-start space-x-3"),
							h.Dt(
								h.Class("mt-0.5"),
								c.TextSROnly("Date"),
								c.IconCalendar()),
							h.Dd(
								g.Text(dateTime))),
						h.Div(
							h.Class("mt-2 flex items-start space-x-3 xl:ml-3.5 xl:mt-0 xl:border-l xl:border-gray-400 xl:border-opacity-50 xl:pl-3.5"),
							h.Dt(
								h.Class("mt-0.5"),
								c.TextSROnly("Location"),
								c.IconMapPin()),
							h.Dd(
								g.Text(e.Location))))
				})),
			c.IconChevronRight(h.Class("text-gray-500 h-8 w-8"))))
}
